@@ -59,7 +59,7 @@ def u(s, encoding = 'utf-8', errors='strict'):
 
 
 class TimblClassifier(object):
-    def __init__(self, fileprefix, timbloptions, format = "Tabbed", dist=True, encoding = 'utf-8', overwrite = True,  flushthreshold=10000):
+    def __init__(self, fileprefix, timbloptions, format = "Tabbed", dist=True, encoding = 'utf-8', overwrite = True,  flushthreshold=10000, threading=False):
         if format.lower() == "tabbed":
             self.format = "Tabbed"
             self.delimiter = "\t"
@@ -83,6 +83,8 @@ class TimblClassifier(object):
         else:
             self.flushed = 1
 
+        self.threading = threading
+
     def validatefeatures(self,features):
         """Returns features in validated form, or raises an Exception. Mostly for internal use"""
         validatedfeatures = []
@@ -143,6 +145,8 @@ class TimblClassifier(object):
         self.api.learn(b(trainfile))
         if save:
             self.save()
+        if self.threading:
+            self.api.initthreading()
 
     def save(self):
         if not self.api:
@@ -150,7 +154,7 @@ class TimblClassifier(object):
         self.api.writeInstanceBase(b(self.fileprefix + ".ibase"))
         self.api.saveWeights(b(self.fileprefix + ".wgt"))
 
-    def classify(self, features):
+    def classify(self, features, allowtopdistribution=True):
 
         features = self.validatefeatures(features)
 
@@ -158,11 +162,18 @@ class TimblClassifier(object):
             self.load()
         testinstance = self.delimiter.join(features) + self.delimiter + "?"
         if self.dist:
-            result, cls, distribution, distance = self.api.classify3(b(testinstance))
+            if self.threading:
+                result, cls, distribution, distance = self.api.classify3safe(b(testinstance), int(not allowtopdistribution))
+            else:
+                result, cls, distribution, distance = self.api.classify3(b(testinstance), int(not allowtopdistribution))
             if result:
                 cls = u(cls)
-                distribution = u(distribution)
-                return (cls, self._parsedistribution(distribution.split(' ')), distance)
+                return (cls, distribution, distance)
+                #distribution = u(distribution)
+                #if cls:
+                #    return (cls, self._parsedistribution(distribution.split(' ')), distance)
+                #else:
+                #   return (cls, {}, distance)
             else:
                 raise ClassifyException("Failed to classify: " + u(testinstance))
         else:
@@ -191,6 +202,8 @@ class TimblClassifier(object):
         self.api.getInstanceBase(b(self.fileprefix + '.ibase'))
         #if os.path.exists(self.fileprefix + ".wgt"):
         #    self.api.getWeights(self.fileprefix + '.wgt')
+        if self.threading:
+            self.api.initthreading()
 
     def addinstance(self, testfile, features, classlabel="?"):
         """Adds an instance to a specific file. Especially suitable for generating test files"""
